### Install packages and load libraries ####

#install.packages(c("geomorph")) # uncomment this line if you need to install or update any of the packages necessary to run this script

library(geomorph)

#change working directory to one of the subfolders
setwd("LDA")

###### Prep Landmark Datasets Output files for LDA analysis #######

## Note: this code only needs to be run once. After output files are generated, LDA analyses can be 
# be run using "LDA_analysis.R" code 


# Dinolite_NoTilt_EO_T1
data1<- readland.tps("Dinolite_NoTilt_EO_T1_Dep1.tps", specID = "imageID") # Read in TPS file
# Files contain the same 247 extant Microtus individuals analyzed in "ProcANOVA.R" but with landmarks of 31
# fossil individuals from Rancho La Brea Project 23, Deposit 1, appended for LDA species classification 
Y.gpa1 <- gpagen(data1) # perform gpa
classifier <- read.csv("Dinolite_Dep1_classifier.csv", header=T)
classifier$species # assign species-group classifiers
coords1 <- as.data.frame(Y.gpa1[["data"]]) # convert gpa file to landmark coordinate data frame
coords_class_1 <- cbind(classifier, coords1) # combine landmark coordinates with species-group classifiers

write.csv(coords_class_1[1:247,], 'Output/Dinolite_NoTilt_EO_T1.csv') # save extant Microtus LDA training set (n = 247)
write.csv(coords_class_1[248:278,], 'Output/Dinolite_NoTilt_EO_T1_Dep1.csv') # save fossil (unknown) Microtus dataset for LDA classification (n = 31) 


# Dinolite_NoTilt_EO_T2
data2<- readland.tps("Dinolite_NoTilt_EO_T2_Dep1.tps", specID = "imageID")
Y.gpa2 <- gpagen(data2)
classifier <- read.csv("Dinolite_Dep1_classifier.csv", header=T)
classifier$species 
coords2 <- as.data.frame(Y.gpa2[["data"]])
coords_class_2 <- cbind(classifier, coords2)

write.csv(coords_class_2[1:247,], 'Output/Dinolite_NoTilt_EO_T2.csv')
write.csv(coords_class_2[248:278,], 'Output/Dinolite_NoTilt_EO_T2_Dep1.csv')


# Dinolite_NoTilt_NO_T1
data3<- readland.tps("Dinolite_NoTilt_NO_T1_Dep1.tps", specID = "imageID")
Y.gpa3 <- gpagen(data3)
classifier <- read.csv("Dinolite_Dep1_classifier.csv", header=T)
classifier$species 
coords3 <- as.data.frame(Y.gpa3[["data"]])
coords_class_3 <- cbind(classifier, coords3)

write.csv(coords_class_3[1:247,], 'Output/Dinolite_NoTilt_NO_T1.csv')
write.csv(coords_class_3[248:278,], 'Output/Dinolite_NoTilt_NO_T1_Dep1.csv')


# Dinolite_NoTilt_NO_T2
data4<- readland.tps("Dinolite_NoTilt_NO_T2_Dep1.tps", specID = "imageID")
Y.gpa4 <- gpagen(data4)
classifier <- read.csv("Dinolite_Dep1_classifier.csv", header=T)
classifier$species 
coords4 <- as.data.frame(Y.gpa4[["data"]])
coords_class_4 <- cbind(classifier, coords4)

write.csv(coords_class_4[1:247,], 'Output/Dinolite_NoTilt_NO_T2.csv')
write.csv(coords_class_4[248:278,], 'Output/Dinolite_NoTilt_NO_T2_Dep1.csv')

# Dinolite_Tilted_EO_T1
data5<- readland.tps("Dinolite_Tilted_EO_T1_Dep1.tps", specID = "imageID")
Y.gpa5 <- gpagen(data5)
classifier <- read.csv("Dinolite_Dep1_classifier.csv", header=T)
classifier$species 
coords5 <- as.data.frame(Y.gpa5[["data"]])
coords_class_5 <- cbind(classifier, coords5)

write.csv(coords_class_5[1:247,], 'Output/Dinolite_Tilted_EO_T1.csv')
write.csv(coords_class_5[248:278,], 'Output/Dinolite_Tilted_EO_T1_Dep1.csv')


#Nikon_NoTilt_EO_T1
data11 <- readland.tps("Nikon_NoTilt_EO_T1_Dep1.tps", specID = "imageID")
Y.gpa11 <- gpagen(data11)
classifier <- read.csv("Nikon_Dep1_classifier.csv", header=T)
classifier$species 
coords11 <- as.data.frame(Y.gpa11[["data"]])
coords_class_11 <- cbind(classifier, coords11)

write.csv(coords_class_11[1:247,], 'Output/Nikon_NoTilt_EO_T1.csv')
write.csv(coords_class_11[248:278,], 'Output/Nikon_NoTilt_EO_T1_Dep1.csv')


#Nikon_NoTilt_EO_T2
data12<- readland.tps("Nikon_NoTilt_EO_T2_Dep1.tps", specID = "imageID")
Y.gpa12 <- gpagen(data12)
classifier <- read.csv("Nikon_Dep1_classifier.csv", header=T)
classifier$species 
coords12 <- as.data.frame(Y.gpa12[["data"]])
coords_class_12 <- cbind(classifier, coords12)

write.csv(coords_class_12[1:247,], 'Output/Nikon_NoTilt_EO_T2.csv')
write.csv(coords_class_12[248:278,], 'Output/Nikon_NoTilt_EO_T2_Dep1.csv')


#Full_Nikon_NoTilt_NO_T1 
data13<- readland.tps("Nikon_NoTilt_NO_T1_Dep1.tps", specID = "imageID")
Y.gpa13 <- gpagen(data13)
classifier <- read.csv("Nikon_Dep1_classifier.csv", header=T)
classifier$species 
coords13 <- as.data.frame(Y.gpa13[["data"]])
coords_class_13 <- cbind(classifier, coords13)

write.csv(coords_class_13[1:247,], 'Output/Nikon_NoTilt_NO_T1.csv')
write.csv(coords_class_13[248:278,], 'Output/Nikon_NoTilt_NO_T1_Dep1.csv')


#Nikon_NoTilt_NO_T2
data14<- readland.tps("Nikon_NoTilt_NO_T2_Dep1.tps", specID = "imageID")
Y.gpa14 <- gpagen(data14)
classifier <- read.csv("Nikon_Dep1_classifier.csv", header=T)
classifier$species 
coords14 <- as.data.frame(Y.gpa14[["data"]])
coords_class_14 <- cbind(classifier, coords14)

write.csv(coords_class_14[1:247,], 'Output/Nikon_NoTilt_NO_T2.csv')
write.csv(coords_class_14[248:278,], 'Output/Nikon_NoTilt_NO_T2_Dep1.csv')

# reset working directory back to the main "Microtus_submission_analysis" folder, which is up one level from "LDA"
setwd("..")
