### Install packages and load libraries ####

#install.packages(c("geomorph")) # uncomment this line if you need to install or update any of the packages necessary to run this script

library(geomorph)

### Read in TPS files----

d1 <- readland.tps("ProcANOVA/Dinolite_NoTilt_NO_T1.tps", specID = "imageID")
d2 <- readland.tps("ProcANOVA/Dinolite_NoTilt_NO_T2.tps", specID = "imageID")
d3 <- readland.tps("ProcANOVA/Dinolite_NoTilt_EO_T1.tps", specID = "imageID")
d4 <- readland.tps("ProcANOVA/Dinolite_NoTilt_EO_T2.tps", specID = "imageID")
d5 <- readland.tps("ProcANOVA/Nikon_NoTilt_NO_T1.tps", specID = "imageID")
d6 <- readland.tps("ProcANOVA/Nikon_NoTilt_NO_T2.tps", specID = "imageID")
d7 <- readland.tps("ProcANOVA/Nikon_NoTilt_EO_T1.tps", specID = "imageID")
d8 <- readland.tps("ProcANOVA/Nikon_NoTilt_EO_T2.tps", specID = "imageID")
d9 <- readland.tps("ProcANOVA/Dinolite_Tilted_EO_T1.tps", specID = "imageID")

### Perform GPA on all datasets ###

# Original dataset names are listed next to each gpa file for pairwise analysis reference. 
# These file names and numbers match each of the landmark data frames below
# (e.g., "Dinolite_NoTilt_EO_T1" = "d3.gpa" = "d3_coords")

d1.gpa <-gpagen(d1)#Dinolite_NoTilt_NO_T1
d2.gpa <-gpagen(d2)#Dinolite_NoTilt_NO_T2 
d3.gpa <- gpagen(d3)#Dinolite_NoTilt_EO_T1
d4.gpa <- gpagen(d4)#Dinolite_NoTilt_EO_T2
d5.gpa <- gpagen(d5)#Nikon_NoTilt_NO_T1
d6.gpa <- gpagen(d6)#Nikon_NoTilt_NO_T2
d7.gpa <- gpagen(d7)#Nikon_NoTilt_EO_T1
d8.gpa <- gpagen(d8)#Nikon_NoTilt_EO_T2
d9.gpa <- gpagen(d9)#Dinolite_Tilted_EO_T1


# Convert LM coords from gpa files to 2D arrays 

d1_coords <- two.d.array(d1.gpa$coords)#Dinolite_NoTilt_NO_T1
d2_coords <- two.d.array(d2.gpa$coords)#Dinolite_NoTilt_NO_T2 
d3_coords <- two.d.array(d3.gpa$coords)#Dinolite_NoTilt_EO_T1
d4_coords <- two.d.array(d4.gpa$coords)#Dinolite_NoTilt_EO_T2
d5_coords <- two.d.array(d5.gpa$coords)#Nikon_NoTilt_NO_T1
d6_coords <- two.d.array(d6.gpa$coords)#Nikon_NoTilt_NO_T2
d7_coords <- two.d.array(d7.gpa$coords)#Nikon_NoTilt_EO_T1
d8_coords <- two.d.array(d8.gpa$coords)#Nikon_NoTilt_EO_T2
d9_coords <- two.d.array(d9.gpa$coords)#Dinolite_Tilted_EO_T1

## Fix ID inconsistencies b/w Dino-lite and Nikon image ID names

# Remove "MX" from Dino-lite IDs
d1_indiv<- rownames(d1_coords)
d1_indiv<- sub('..', '', d1_indiv)
d2_indiv<- rownames(d2_coords)
d2_indiv<- sub('..', '', d2_indiv)
d3_indiv<- rownames(d3_coords)
d3_indiv<- sub('..', '', d3_indiv)
d4_indiv<- rownames(d4_coords)
d4_indiv<- sub('..', '', d4_indiv)
d9_indiv<- rownames(d9_coords)
d9_indiv<- sub('..', '', d9_indiv)

# Replace "LL" with "L" in Dino-lite and Nikon IDs
d1_indiv<- gsub('LL', 'L', d1_indiv)
d2_indiv<- gsub('LL', 'L', d2_indiv)
d3_indiv<- gsub('LL', 'L', d3_indiv)
d4_indiv<- gsub('LL', 'L', d4_indiv)
d5_indiv <- rownames(d5_coords)
d5_indiv<- gsub('LL', 'L', d5_indiv)
d6_indiv <- rownames(d6_coords)
d6_indiv<- gsub('LL', 'L', d6_indiv)
d7_indiv <- rownames(d7_coords)
d7_indiv<- gsub('LL', 'L', d7_indiv)
d8_indiv <- rownames(d8_coords)
d8_indiv<- gsub('LL', 'L', d8_indiv)
d9_indiv<- gsub('LL', 'L', d9_indiv)
d9_indiv<- gsub('LA', 'L', d9_indiv)


####ProcANOVA analyses####

# Nested hierarchical model to quantify error attributed to biological variation (species and individuals),
# imagining devices, different observers, and within observers.
# WARNING - this may require substantial computing time. Reduce iterations to speed up computations.

LMdata_all <- rbind(d1_coords, d2_coords, d3_coords, d4_coords, d5_coords, 
                    d6_coords, d7_coords, d8_coords)

classifier <- read.csv("ProcANOVA/Full_classifier.csv", header=T)
species <-classifier[,1]

individuals <- c(d1_indiv, d2_indiv, d3_indiv, d4_indiv, d5_indiv, d6_indiv,
                 d7_indiv, d8_indiv)

device <- c(rep("Dinolite", nrow(d1_coords)), rep("Dinolite", nrow(d2_coords)), 
            rep("Dinolite", nrow(d3_coords)), rep("Dinolite", nrow(d4_coords)),
            rep("Nikon", nrow(d5_coords)), rep("Nikon", nrow(d6_coords)),
            rep("Nikon", nrow(d7_coords)), rep("Nikon", nrow(d8_coords)))

interobservers <- c(rep("NO", nrow(d1_coords)), rep("NO", nrow(d2_coords)), 
                    rep("EO", nrow(d3_coords)), rep("EO", nrow(d4_coords)),
                    rep("NO", nrow(d5_coords)), rep("NO", nrow(d6_coords)),
                    rep("EO", nrow(d7_coords)), rep("EO", nrow(d8_coords)))

intraobservers <- c(rep("T1", nrow(d1_coords)), rep("T2", nrow(d2_coords)), 
                    rep("T1", nrow(d3_coords)), rep("T2", nrow(d4_coords)),
                    rep("T1", nrow(d5_coords)), rep("T2", nrow(d6_coords)),
                    rep("T1", nrow(d7_coords)), rep("T2", nrow(d8_coords)))


Nested_ANOVA <- procD.lm(LMdata_all ~ species/individuals/device/
                           interobservers/intraobservers, seed = NULL, RRPP = TRUE, iter=999) 
#reduce iterations to quicken
summary(Nested_ANOVA)


### Pairwise analyses and repeatability calculations ###
# because we are using procrustes anova, use the mean squares not the sum of squares for repeatability
# See Arnqvist and Mårtensson (1998), Fruciano (2016)

## Within Observer Error

#Dinolite_NoTilt_NO_T1 versus Dinolite_NoTilt_NO_T2
individuals <- c(d1_indiv, d2_indiv)
LMdata_all <- rbind(d1_coords, d2_coords)
pairwise.1 <- procD.lm(LMdata_all~individuals,iter=999)
summary(pairwise.1)

#pairwise repeatibility ----
#Dinolite_NoTilt_NO_T1 versus Dinolite_NoTilt_NO_T2
n_1 <- 2 
S2among <- (pairwise.1$aov.table$MS[1] - pairwise.1$aov.table$MS[2]) / n_1
S2within <- pairwise.1$aov.table$MS[2]
R_1 <- S2among/(S2within + S2among)
Rn_1 <- (n_1*R_1)/(1+(n_1-1)/R_1)
Rn_1


#Dinolite_NoTilt_EO_T1 versus Dinolite_NoTilt_EO_T2
individuals <- c(d3_indiv, d4_indiv)
LMdata_all <- rbind(d3_coords, d4_coords)
pairwise.2 <- procD.lm(LMdata_all~individuals,iter=999)
summary(pairwise.2)

n_2 <- 2 
S2among <- (pairwise.2$aov.table$MS[1] - pairwise.2$aov.table$MS[2]) / n_2
S2within <- pairwise.2$aov.table$MS[2]
R_2 <- S2among/(S2within + S2among)
Rn_2 <- (n_2*R_2)/(1+(n_2-1)/R_2)
Rn_2

#Nikon_NoTilt_NO_T1 versus Nikon_NoTilt_NO_T2
individuals <- c(d5_indiv, d6_indiv)
LMdata_all <- rbind(d5_coords, d6_coords)
pairwise.3 <- procD.lm(LMdata_all~individuals,iter=999)
summary(pairwise.3)

n_3 <- 2 
S2among <- (pairwise.3$aov.table$MS[1] - pairwise.3$aov.table$MS[2]) / n_3
S2within <- pairwise.3$aov.table$MS[2]
R_3 <- S2among/(S2within + S2among)
Rn_3 <- (n_3*R_3)/(1+(n_3-1)/R_3)
Rn_3

#Nikon_NoTilt_EO_T1 versus Nikon_NoTilt_EO_T2
individuals <- c(d7_indiv, d8_indiv)
LMdata_all <- rbind(d7_coords, d8_coords)
pairwise.4 <- procD.lm(LMdata_all~individuals,iter=999)
summary(pairwise.4)

n_4 <- 2 
S2among <- (pairwise.4$aov.table$MS[1] - pairwise.4$aov.table$MS[2]) / n_4
S2within <- pairwise.4$aov.table$MS[2]
R_4 <- S2among/(S2within + S2among)
Rn_4 <- (n_4*R_4)/(1+(n_4-1)/R_4)
Rn_4

#Mean Intraobserver repeatability:
(Rn_1+Rn_2+Rn_3+Rn_4)/4


## Between Observer Error

#Dinolite_NoTilt_NO_T1 versus Dinolite_NoTilt_EO_T1
individuals <- c(d1_indiv, d3_indiv)
LMdata_all <- rbind(d1_coords, d3_coords)
pairwise.5 <- procD.lm(LMdata_all~individuals,iter=999)
summary(pairwise.5)

n_5 <- 2 
S2among <- (pairwise.5$aov.table$MS[1] - pairwise.5$aov.table$MS[2]) / n_5
S2within <- pairwise.5$aov.table$MS[2]
R_5 <- S2among/(S2within + S2among)
Rn_5 <- (n_5*R_5)/(1+(n_5-1)/R_5)
Rn_5

#Dinolite_NoTilt_NO_T1 versus Dinolite_NoTilt_EO_T2
individuals <- c(d1_indiv, d4_indiv)
LMdata_all <- rbind(d1_coords, d4_coords)
pairwise.6 <- procD.lm(LMdata_all~individuals,iter=999)
summary(pairwise.6)

n_6 <- 2 
S2among <- (pairwise.6$aov.table$MS[1] - pairwise.6$aov.table$MS[2]) / n_6
S2within <- pairwise.6$aov.table$MS[2]
R_6 <- S2among/(S2within + S2among)
Rn_6 <- (n_6*R_6)/(1+(n_6-1)/R_6)
Rn_6

#Dinolite_NoTilt_NO_T2 versus Dinolite_NoTilt_EO_T1
individuals <- c(d2_indiv, d3_indiv)
LMdata_all <- rbind(d2_coords, d3_coords)
pairwise.7 <- procD.lm(LMdata_all~individuals,iter=999)
summary(pairwise.7)

n_7 <- 2 
S2among <- (pairwise.7$aov.table$MS[1] - pairwise.7$aov.table$MS[2]) / n_7
S2within <- pairwise.7$aov.table$MS[2]
R_7 <- S2among/(S2within + S2among)
Rn_7 <- (n_7*R_7)/(1+(n_7-1)/R_7)
Rn_7

#Dinolite_NoTilt_NO_T2 versus Dinolite_NoTilt_EO_T2
individuals <- c(d2_indiv, d4_indiv)
LMdata_all <- rbind(d2_coords, d4_coords)
pairwise.8 <- procD.lm(LMdata_all~individuals,iter=999)
summary(pairwise.8)

n_8 <- 2 
S2among <- (pairwise.8$aov.table$MS[1] - pairwise.8$aov.table$MS[2]) / n_8
S2within <- pairwise.8$aov.table$MS[2]
R_8 <- S2among/(S2within + S2among)
Rn_8 <- (n_8*R_8)/(1+(n_8-1)/R_8)
Rn_8

#Nikon_NoTilt_NO_T1 versus Nikon_NoTilt_EO_T1
individuals <- c(d5_indiv, d7_indiv)
LMdata_all <- rbind(d5_coords, d7_coords)
pairwise.9 <- procD.lm(LMdata_all~individuals,iter=999)
summary(pairwise.9)

n_9 <- 2 
S2among <- (pairwise.9$aov.table$MS[1] - pairwise.9$aov.table$MS[2]) / n_9
S2within <- pairwise.9$aov.table$MS[2]
R_9 <- S2among/(S2within + S2among)
Rn_9 <- (n_9*R_9)/(1+(n_9-1)/R_9)
Rn_9

#Nikon_NoTilt_NO_T1 versus Nikon_NoTilt_EO_T2
individuals <- c(d5_indiv, d8_indiv)
LMdata_all <- rbind(d5_coords, d8_coords)
pairwise.10 <- procD.lm(LMdata_all~individuals,iter=999)
summary(pairwise.10)

n_10 <- 2 
S2among <- (pairwise.10$aov.table$MS[1] - pairwise.10$aov.table$MS[2]) / n_10
S2within <- pairwise.10$aov.table$MS[2]
R_10 <- S2among/(S2within + S2among)
Rn_10 <- (n_10*R_10)/(1+(n_10-1)/R_10)
Rn_10

#Nikon_NoTilt_NO_T2 versus Nikon_NoTilt_EO_T1
individuals <- c(d6_indiv, d7_indiv)
LMdata_all <- rbind(d6_coords, d7_coords)
pairwise.11 <- procD.lm(LMdata_all~individuals,iter=999)
summary(pairwise.11)

n_11 <- 2 
S2among <- (pairwise.11$aov.table$MS[1] - pairwise.11$aov.table$MS[2]) / n_11
S2within <- pairwise.11$aov.table$MS[2]
R_11 <- S2among/(S2within + S2among)
Rn_11 <- (n_11*R_11)/(1+(n_11-1)/R_11)
Rn_11

#Nikon_NoTilt_NO_T2 versus Nikon_NoTilt_EO_T2
individuals <- c(d6_indiv, d8_indiv)
LMdata_all <- rbind(d6_coords, d8_coords)
pairwise.12 <- procD.lm(LMdata_all~individuals,iter=999)
summary(pairwise.12)

n_12 <- 2 
S2among <- (pairwise.12$aov.table$MS[1] - pairwise.12$aov.table$MS[2]) / n_12
S2within <- pairwise.12$aov.table$MS[2]
R_12 <- S2among/(S2within + S2among)
Rn_12 <- (n_12*R_12)/(1+(n_12-1)/R_12)
Rn_12

#Mean Intraobserver repeatability:
(Rn_5+Rn_6+Rn_7+Rn_8+Rn_9+Rn_10+Rn_11+Rn_12)/8

##Device Error 

#Dinolite_NoTilt_NO_T1 versus Nikon_NoTilt_NO_T1
individuals <- c(d1_indiv, d5_indiv)
LMdata_all <- rbind(d1_coords, d5_coords)
pairwise.13 <- procD.lm(LMdata_all~individuals,iter=999)
summary(pairwise.13)

n_13 <- 2 
S2among <- (pairwise.13$aov.table$MS[1] - pairwise.13$aov.table$MS[2]) / n_13
S2within <- pairwise.13$aov.table$MS[2]
R_13 <- S2among/(S2within + S2among)
Rn_13 <- (n_13*R_13)/(1+(n_13-1)/R_13)
Rn_13

#Dinolite_NoTilt_NO_T1 versus Nikon_NoTilt_NO_T2
individuals <- c(d1_indiv, d6_indiv)
LMdata_all <- rbind(d1_coords, d6_coords)
pairwise.14 <- procD.lm(LMdata_all~individuals,iter=999)
summary(pairwise.14)

n_14 <- 2 
S2among <- (pairwise.14$aov.table$MS[1] - pairwise.14$aov.table$MS[2]) / n_14
S2within <- pairwise.14$aov.table$MS[2]
R_14 <- S2among/(S2within + S2among)
Rn_14 <- (n_14*R_14)/(1+(n_14-1)/R_14)
Rn_14

#Dinolite_NoTilt_NO_T2 versus Nikon_NoTilt_NO_T1
individuals <- c(d2_indiv, d5_indiv)
LMdata_all <- rbind(d2_coords, d5_coords)
pairwise.15 <- procD.lm(LMdata_all~individuals,iter=999)
summary(pairwise.15)

n_15 <- 2 
S2among <- (pairwise.15$aov.table$MS[1] - pairwise.15$aov.table$MS[2]) / n_15
S2within <- pairwise.15$aov.table$MS[2]
R_15 <- S2among/(S2within + S2among)
Rn_15 <- (n_15*R_15)/(1+(n_15-1)/R_15)
Rn_15

#Dinolite_NoTilt_NO_T2 versus Nikon_NoTilt_NO_T2
individuals <- c(d2_indiv, d6_indiv)
LMdata_all <- rbind(d2_coords, d6_coords)
pairwise.16 <- procD.lm(LMdata_all~individuals,iter=999)
summary(pairwise.16)

n_16 <- 2 
S2among <- (pairwise.16$aov.table$MS[1] - pairwise.16$aov.table$MS[2]) / n_16
S2within <- pairwise.16$aov.table$MS[2]
R_16 <- S2among/(S2within + S2among)
Rn_16 <- (n_16*R_16)/(1+(n_16-1)/R_16)
Rn_16

#Dinolite_NoTilt_EO_T1 versus Nikon_NoTilt_EO_T1
individuals <- c(d3_indiv, d7_indiv)
LMdata_all <- rbind(d3_coords, d7_coords)
pairwise.17 <- procD.lm(LMdata_all~individuals,iter=999)
summary(pairwise.17)

n_17 <- 2 
S2among <- (pairwise.17$aov.table$MS[1] - pairwise.17$aov.table$MS[2]) / n_17
S2within <- pairwise.17$aov.table$MS[2]
R_17 <- S2among/(S2within + S2among)
Rn_17 <- (n_17*R_17)/(1+(n_17-1)/R_17)
Rn_17

#Dinolite_NoTilt_EO_T1 versus Nikon_NoTilt_EO_T2
individuals <- c(d3_indiv, d8_indiv)
LMdata_all <- rbind(d3_coords, d8_coords)
pairwise.18 <- procD.lm(LMdata_all~individuals,iter=999)
summary(pairwise.18)

n_18 <- 2 
S2among <- (pairwise.18$aov.table$MS[1] - pairwise.18$aov.table$MS[2]) / n_18
S2within <- pairwise.18$aov.table$MS[2]
R_18 <- S2among/(S2within + S2among)
Rn_18 <- (n_18*R_18)/(1+(n_18-1)/R_18)
Rn_18


#Dinolite_NoTilt_EO_T2 versus Nikon_NoTilt_EO_T1
individuals <- c(d4_indiv, d7_indiv)
LMdata_all <- rbind(d4_coords, d7_coords)
pairwise.19 <- procD.lm(LMdata_all~individuals,iter=999)
summary(pairwise.19)

n_19 <- 2 
S2among <- (pairwise.19$aov.table$MS[1] - pairwise.19$aov.table$MS[2]) / n_19
S2within <- pairwise.19$aov.table$MS[2]
R_19 <- S2among/(S2within + S2among)
Rn_19 <- (n_19*R_19)/(1+(n_19-1)/R_19)
Rn_19

#Dinolite_NoTilt_EO_T2 versus Nikon_NoTilt_EO_T2
individuals <- c(d4_indiv, d8_indiv)
LMdata_all <- rbind(d4_coords, d8_coords)
pairwise.20 <- procD.lm(LMdata_all~individuals,iter=999)
summary(pairwise.20)

n_20 <- 2 
S2among <- (pairwise.20$aov.table$MS[1] - pairwise.20$aov.table$MS[2]) / n_20
S2within <- pairwise.20$aov.table$MS[2]
R_20 <- S2among/(S2within + S2among)
Rn_20 <- (n_20*R_20)/(1+(n_20-1)/R_20)
Rn_20


#Mean Device repeatability:
(Rn_13+Rn_14+Rn_15+Rn_16+Rn_17+Rn_18+Rn_19+Rn_20)/8


### Presentation Error 
#Dinolite_NoTilt_EO_T1 versus Dinolite_Tilted_EO_T1
individuals <- c(d3_indiv, d9_indiv)
LMdata_all <- rbind(d3_coords, d9_coords)
pairwise.21 <- procD.lm(LMdata_all~individuals,iter=999)
summary(pairwise.21)

n_21 <- 2 
S2among <- (pairwise.21$aov.table$MS[1] - pairwise.21$aov.table$MS[2]) / n_21
S2within <- pairwise.21$aov.table$MS[2]
R_21 <- S2among/(S2within + S2among)
Rn_21 <- (n_21*R_21)/(1+(n_21-1)/R_21)
Rn_21

#Dinolite_NoTilt_EO_T2 versus Dinolite_Tilted_EO_T1
individuals <- c(d4_indiv, d9_indiv)
LMdata_all <- rbind(d4_coords, d9_coords)
pairwise.22 <- procD.lm(LMdata_all~individuals,iter=999)
summary(pairwise.22)

n_22 <- 2 
S2among <- (pairwise.22$aov.table$MS[1] - pairwise.22$aov.table$MS[2]) / n_22
S2within <- pairwise.22$aov.table$MS[2]
R_22 <- S2among/(S2within + S2among)
Rn_22 <- (n_22*R_22)/(1+(n_22-1)/R_22)
Rn_22

#Mean Presentation repeatability:
(Rn_21+Rn_22)/2

#References
#Arnqvist, G., Mårtensson, T., 1998. Measurement error in geometric morphometrics: empirical strategies to assess and reduce its impact on measures of shape. Zoologica Academiae Scientiarum Hungaricae 44, 73–96.
#Fruciano, C., 2016. Measurement error in geometric morphometrics. Development Genes and Evolution 226, 139–158. https://doi.org/10.1007/s00427-016-0537-4
